@@ -22,14 +22,10 @@ import org.apache.hudi.cli.HoodieCLI;
 import org.apache.hudi.cli.HoodiePrintHelper;
 import org.apache.hudi.cli.HoodieTableHeaderFields;
 import org.apache.hudi.cli.TableHeader;
-import org.apache.hudi.cli.utils.CommitUtil;
-import org.apache.hudi.cli.utils.InputStreamConsumer;
-import org.apache.hudi.cli.utils.SparkUtil;
 import org.apache.hudi.common.model.HoodieCommitMetadata;
 import org.apache.hudi.common.model.HoodieReplaceCommitMetadata;
 import org.apache.hudi.common.model.HoodieWriteStat;
 import org.apache.hudi.common.table.HoodieTableMetaClient;
-import org.apache.hudi.common.table.timeline.HoodieActiveTimeline;
 import org.apache.hudi.common.table.timeline.HoodieArchivedTimeline;
 import org.apache.hudi.common.table.timeline.HoodieDefaultTimeline;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
@@ -38,187 +34,164 @@ import org.apache.hudi.common.util.NumericUtils;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.StringUtils;
 
-import org.apache.spark.launcher.SparkLauncher;
-import org.springframework.shell.core.CommandMarker;
-import org.springframework.shell.core.annotation.CliCommand;
-import org.springframework.shell.core.annotation.CliOption;
-import org.springframework.stereotype.Component;
+import org.springframework.shell.standard.ShellComponent;
+import org.springframework.shell.standard.ShellMethod;
+import org.springframework.shell.standard.ShellOption;
 
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.function.Function;
 import java.util.stream.Collectors;
 
+import static org.apache.hudi.cli.utils.CommitUtil.getTimeDaysAgo;
+import static org.apache.hudi.common.table.timeline.TimelineUtils.getTimeline;
+
 /**
  * CLI command to display commits options.
  */
-@Component
-public class CommitsCommand implements CommandMarker {
+@ShellComponent
+public class CommitsCommand {
 
   private String printCommits(HoodieDefaultTimeline timeline,
-                              final Integer limit, final String sortByField,
+                              final Integer limit,
+                              final String sortByField,
                               final boolean descending,
                               final boolean headerOnly,
                               final String tempTableName) throws IOException {
     final List<Comparable[]> rows = new ArrayList<>();
 
     final List<HoodieInstant> commits = timeline.getCommitsTimeline().filterCompletedInstants()
-            .getInstants().collect(Collectors.toList());
-    // timeline can be read from multiple files. So sort is needed instead of reversing the collection
-    Collections.sort(commits, HoodieInstant.COMPARATOR.reversed());
+        .getInstants().sorted(HoodieInstant.COMPARATOR.reversed()).collect(Collectors.toList());
 
-    for (int i = 0; i < commits.size(); i++) {
-      final HoodieInstant commit = commits.get(i);
-      final HoodieCommitMetadata commitMetadata = HoodieCommitMetadata.fromBytes(
-              timeline.getInstantDetails(commit).get(),
-              HoodieCommitMetadata.class);
-      rows.add(new Comparable[]{commit.getTimestamp(),
-              commitMetadata.fetchTotalBytesWritten(),
-              commitMetadata.fetchTotalFilesInsert(),
-              commitMetadata.fetchTotalFilesUpdated(),
-              commitMetadata.fetchTotalPartitionsWritten(),
-              commitMetadata.fetchTotalRecordsWritten(),
-              commitMetadata.fetchTotalUpdateRecordsWritten(),
-              commitMetadata.fetchTotalWriteErrors()});
+    for (final HoodieInstant commit : commits) {
+      if (timeline.getInstantDetails(commit).isPresent()) {
+        final HoodieCommitMetadata commitMetadata = HoodieCommitMetadata.fromBytes(
+            timeline.getInstantDetails(commit).get(),
+            HoodieCommitMetadata.class);
+        rows.add(new Comparable[] {commit.getTimestamp(),
+            commitMetadata.fetchTotalBytesWritten(),
+            commitMetadata.fetchTotalFilesInsert(),
+            commitMetadata.fetchTotalFilesUpdated(),
+            commitMetadata.fetchTotalPartitionsWritten(),
+            commitMetadata.fetchTotalRecordsWritten(),
+            commitMetadata.fetchTotalUpdateRecordsWritten(),
+            commitMetadata.fetchTotalWriteErrors()});
+      }
     }
 
     final Map<String, Function<Object, String>> fieldNameToConverterMap = new HashMap<>();
-    fieldNameToConverterMap.put(HoodieTableHeaderFields.HEADER_TOTAL_BYTES_WRITTEN, entry -> {
-      return NumericUtils.humanReadableByteCount((Double.valueOf(entry.toString())));
-    });
+    fieldNameToConverterMap.put(
+        HoodieTableHeaderFields.HEADER_TOTAL_BYTES_WRITTEN,
+        entry -> NumericUtils.humanReadableByteCount((Double.parseDouble(entry.toString()))));
 
-    final TableHeader header = new TableHeader()
-            .addTableHeaderField(HoodieTableHeaderFields.HEADER_COMMIT_TIME)
-            .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_BYTES_WRITTEN)
-            .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_FILES_ADDED)
-            .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_FILES_UPDATED)
-            .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_PARTITIONS_WRITTEN)
-            .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_RECORDS_WRITTEN)
-            .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_UPDATE_RECORDS_WRITTEN)
-            .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_ERRORS);
+    final TableHeader header = HoodieTableHeaderFields.getTableHeader();
 
     return HoodiePrintHelper.print(header, fieldNameToConverterMap, sortByField, descending,
-            limit, headerOnly, rows, tempTableName);
+        limit, headerOnly, rows, tempTableName);
   }
 
   private String printCommitsWithMetadata(HoodieDefaultTimeline timeline,
-                              final Integer limit, final String sortByField,
-                              final boolean descending,
-                              final boolean headerOnly,
-                              final String tempTableName) throws IOException {
+                                          final Integer limit, final String sortByField,
+                                          final boolean descending,
+                                          final boolean headerOnly,
+                                          final String tempTableName,
+                                          final String partition) throws IOException {
     final List<Comparable[]> rows = new ArrayList<>();
 
     final List<HoodieInstant> commits = timeline.getCommitsTimeline().filterCompletedInstants()
-            .getInstants().collect(Collectors.toList());
-    // timeline can be read from multiple files. So sort is needed instead of reversing the collection
-    Collections.sort(commits, HoodieInstant.COMPARATOR.reversed());
+        .getInstants().sorted(HoodieInstant.COMPARATOR.reversed()).collect(Collectors.toList());
 
-    for (int i = 0; i < commits.size(); i++) {
-      final HoodieInstant commit = commits.get(i);
-      final HoodieCommitMetadata commitMetadata = HoodieCommitMetadata.fromBytes(
-              timeline.getInstantDetails(commit).get(),
-              HoodieCommitMetadata.class);
+    for (final HoodieInstant commit : commits) {
+      if (timeline.getInstantDetails(commit).isPresent()) {
+        final HoodieCommitMetadata commitMetadata = HoodieCommitMetadata.fromBytes(
+            timeline.getInstantDetails(commit).get(),
+            HoodieCommitMetadata.class);
 
-      for (Map.Entry<String, List<HoodieWriteStat>> partitionWriteStat :
-              commitMetadata.getPartitionToWriteStats().entrySet()) {
-        for (HoodieWriteStat hoodieWriteStat : partitionWriteStat.getValue()) {
-          rows.add(new Comparable[]{ commit.getAction(), commit.getTimestamp(), hoodieWriteStat.getPartitionPath(),
+        for (Map.Entry<String, List<HoodieWriteStat>> partitionWriteStat :
+            commitMetadata.getPartitionToWriteStats().entrySet()) {
+          for (HoodieWriteStat hoodieWriteStat : partitionWriteStat.getValue()) {
+            if (StringUtils.isNullOrEmpty(partition) || partition.equals(hoodieWriteStat.getPartitionPath())) {
+              rows.add(new Comparable[] {commit.getAction(), commit.getTimestamp(), hoodieWriteStat.getPartitionPath(),
                   hoodieWriteStat.getFileId(), hoodieWriteStat.getPrevCommit(), hoodieWriteStat.getNumWrites(),
                   hoodieWriteStat.getNumInserts(), hoodieWriteStat.getNumDeletes(),
                   hoodieWriteStat.getNumUpdateWrites(), hoodieWriteStat.getTotalWriteErrors(),
                   hoodieWriteStat.getTotalLogBlocks(), hoodieWriteStat.getTotalCorruptLogBlock(),
                   hoodieWriteStat.getTotalRollbackBlocks(), hoodieWriteStat.getTotalLogRecords(),
                   hoodieWriteStat.getTotalUpdatedRecordsCompacted(), hoodieWriteStat.getTotalWriteBytes()
-          });
+              });
+            }
+          }
         }
       }
     }
 
     final Map<String, Function<Object, String>> fieldNameToConverterMap = new HashMap<>();
-    fieldNameToConverterMap.put(HoodieTableHeaderFields.HEADER_TOTAL_BYTES_WRITTEN, entry -> {
-      return NumericUtils.humanReadableByteCount((Double.valueOf(entry.toString())));
-    });
+    fieldNameToConverterMap.put(
+        HoodieTableHeaderFields.HEADER_TOTAL_BYTES_WRITTEN,
+        entry -> NumericUtils.humanReadableByteCount((Double.parseDouble(entry.toString()))));
 
-    TableHeader header = new TableHeader().addTableHeaderField(HoodieTableHeaderFields.HEADER_ACTION)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_INSTANT)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_PARTITION)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_FILE_ID)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_PREVIOUS_COMMIT)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_NUM_WRITES)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_NUM_INSERTS)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_NUM_DELETES)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_NUM_UPDATE_WRITES)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_ERRORS)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_LOG_BLOCKS)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_CORRUPT_LOG_BLOCKS)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_ROLLBACK_BLOCKS)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_LOG_RECORDS)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_UPDATED_RECORDS_COMPACTED)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_BYTES_WRITTEN);
-
-    return HoodiePrintHelper.print(header, new HashMap<>(), sortByField, descending,
-            limit, headerOnly, rows, tempTableName);
+    return HoodiePrintHelper.print(HoodieTableHeaderFields.getTableHeaderWithExtraMetadata(),
+        fieldNameToConverterMap, sortByField, descending, limit, headerOnly, rows, tempTableName);
   }
 
-  @CliCommand(value = "commits show", help = "Show the commits")
+  @ShellMethod(key = "commits show", value = "Show the commits")
   public String showCommits(
-      @CliOption(key = {"includeExtraMetadata"}, help = "Include extra metadata",
-          unspecifiedDefaultValue = "false") final boolean includeExtraMetadata,
-      @CliOption(key = {"createView"}, mandatory = false, help = "view name to store output table",
-          unspecifiedDefaultValue = "") final String exportTableName,
-      @CliOption(key = {"limit"}, help = "Limit commits",
-          unspecifiedDefaultValue = "-1") final Integer limit,
-      @CliOption(key = {"sortBy"}, help = "Sorting Field", unspecifiedDefaultValue = "") final String sortByField,
-      @CliOption(key = {"desc"}, help = "Ordering", unspecifiedDefaultValue = "false") final boolean descending,
-      @CliOption(key = {"headeronly"}, help = "Print Header Only",
-          unspecifiedDefaultValue = "false") final boolean headerOnly)
+      @ShellOption(value = {"--includeExtraMetadata"}, help = "Include extra metadata",
+          defaultValue = "false") final boolean includeExtraMetadata,
+      @ShellOption(value = {"--createView"}, help = "view name to store output table",
+              defaultValue = "") final String exportTableName,
+      @ShellOption(value = {"--limit"}, help = "Limit commits",
+              defaultValue = "-1") final Integer limit,
+      @ShellOption(value = {"--sortBy"}, help = "Sorting Field", defaultValue = "") final String sortByField,
+      @ShellOption(value = {"--desc"}, help = "Ordering", defaultValue = "false") final boolean descending,
+      @ShellOption(value = {"--headeronly"}, help = "Print Header Only",
+              defaultValue = "false") final boolean headerOnly,
+      @ShellOption(value = {"--partition"}, help = "Partition value", defaultValue = ShellOption.NULL) final String partition,
+      @ShellOption(value = {"--includeArchivedTimeline"}, help = "Include archived commits as well",
+              defaultValue = "false") final boolean includeArchivedTimeline)
       throws IOException {
 
-    HoodieActiveTimeline activeTimeline = HoodieCLI.getTableMetaClient().getActiveTimeline();
+    HoodieDefaultTimeline timeline = getTimeline(HoodieCLI.getTableMetaClient(), includeArchivedTimeline);
     if (includeExtraMetadata) {
-      return printCommitsWithMetadata(activeTimeline, limit, sortByField, descending, headerOnly, exportTableName);
-    } else  {
-      return printCommits(activeTimeline, limit, sortByField, descending, headerOnly, exportTableName);
+      return printCommitsWithMetadata(timeline, limit, sortByField, descending, headerOnly, exportTableName, partition);
+    } else {
+      return printCommits(timeline, limit, sortByField, descending, headerOnly, exportTableName);
     }
   }
 
-  @CliCommand(value = "commits showarchived", help = "Show the archived commits")
+  @ShellMethod(key = "commits showarchived", value = "Show the archived commits")
   public String showArchivedCommits(
-          @CliOption(key = {"includeExtraMetadata"}, help = "Include extra metadata",
-                  unspecifiedDefaultValue = "false") final boolean includeExtraMetadata,
-          @CliOption(key = {"createView"}, mandatory = false, help = "view name to store output table",
-                  unspecifiedDefaultValue = "") final String exportTableName,
-          @CliOption(key = {"startTs"},  mandatory = false, help = "start time for commits, default: now - 10 days")
+      @ShellOption(value = {"--includeExtraMetadata"}, help = "Include extra metadata",
+          defaultValue = "false") final boolean includeExtraMetadata,
+      @ShellOption(value = {"--createView"}, help = "view name to store output table",
+          defaultValue = "") final String exportTableName,
+      @ShellOption(value = {"--startTs"}, defaultValue = ShellOption.NULL, help = "start time for commits, default: now - 10 days")
           String startTs,
-          @CliOption(key = {"endTs"},  mandatory = false, help = "end time for commits, default: now - 1 day")
+      @ShellOption(value = {"--endTs"}, defaultValue = ShellOption.NULL, help = "end time for commits, default: now - 1 day")
           String endTs,
-          @CliOption(key = {"limit"}, mandatory = false, help = "Limit commits", unspecifiedDefaultValue = "-1")
-          final Integer limit,
-          @CliOption(key = {"sortBy"}, help = "Sorting Field", unspecifiedDefaultValue = "")
-          final String sortByField,
-          @CliOption(key = {"desc"}, help = "Ordering", unspecifiedDefaultValue = "false")
-          final boolean descending,
-          @CliOption(key = {"headeronly"}, help = "Print Header Only", unspecifiedDefaultValue = "false")
-          final boolean headerOnly)
-          throws IOException {
+      @ShellOption(value = {"--limit"}, help = "Limit commits", defaultValue = "-1") final Integer limit,
+      @ShellOption(value = {"--sortBy"}, help = "Sorting Field", defaultValue = "") final String sortByField,
+      @ShellOption(value = {"--desc"}, help = "Ordering", defaultValue = "false") final boolean descending,
+      @ShellOption(value = {"--headeronly"}, help = "Print Header Only", defaultValue = "false") final boolean headerOnly,
+      @ShellOption(value = {"--partition"}, help = "Partition value", defaultValue = ShellOption.NULL) final String partition)
+      throws IOException {
     if (StringUtils.isNullOrEmpty(startTs)) {
-      startTs = CommitUtil.getTimeDaysAgo(10);
+      startTs = getTimeDaysAgo(10);
     }
     if (StringUtils.isNullOrEmpty(endTs)) {
-      endTs = CommitUtil.getTimeDaysAgo(1);
+      endTs = getTimeDaysAgo(1);
     }
     HoodieArchivedTimeline archivedTimeline = HoodieCLI.getTableMetaClient().getArchivedTimeline();
     try {
       archivedTimeline.loadInstantDetailsInMemory(startTs, endTs);
       HoodieDefaultTimeline timelineRange = archivedTimeline.findInstantsInRange(startTs, endTs);
       if (includeExtraMetadata) {
-        return printCommitsWithMetadata(timelineRange, limit, sortByField, descending, headerOnly, exportTableName);
-      } else  {
+        return printCommitsWithMetadata(timelineRange, limit, sortByField, descending, headerOnly, exportTableName, partition);
+      } else {
         return printCommits(timelineRange, limit, sortByField, descending, headerOnly, exportTableName);
       }
     } finally {
@@ -227,50 +200,22 @@ public class CommitsCommand implements CommandMarker {
     }
   }
 
-  @CliCommand(value = "commit rollback", help = "Rollback a commit")
-  public String rollbackCommit(@CliOption(key = {"commit"}, help = "Commit to rollback") final String instantTime,
-      @CliOption(key = {"sparkProperties"}, help = "Spark Properties File Path") final String sparkPropertiesPath,
-      @CliOption(key = "sparkMaster", unspecifiedDefaultValue = "", help = "Spark Master") String master,
-      @CliOption(key = "sparkMemory", unspecifiedDefaultValue = "4G",
-         help = "Spark executor memory") final String sparkMemory,
-      @CliOption(key = "rollbackUsingMarkers", unspecifiedDefaultValue = "false",
-         help = "Enabling marker based rollback") final String rollbackUsingMarkers)
-      throws Exception {
-    HoodieActiveTimeline activeTimeline = HoodieCLI.getTableMetaClient().getActiveTimeline();
-    HoodieTimeline completedTimeline = activeTimeline.getCommitsTimeline().filterCompletedInstants();
-    HoodieTimeline filteredTimeline = completedTimeline.filter(instant -> instant.getTimestamp().equals(instantTime));
-    if (filteredTimeline.empty()) {
-      return "Commit " + instantTime + " not found in Commits " + completedTimeline;
-    }
-
-    SparkLauncher sparkLauncher = SparkUtil.initLauncher(sparkPropertiesPath);
-    sparkLauncher.addAppArgs(SparkMain.SparkCommand.ROLLBACK.toString(), master, sparkMemory, instantTime,
-        HoodieCLI.getTableMetaClient().getBasePath(), rollbackUsingMarkers);
-    Process process = sparkLauncher.launch();
-    InputStreamConsumer.captureOutput(process);
-    int exitCode = process.waitFor();
-    // Refresh the current
-    HoodieCLI.refreshTableMetadata();
-    if (exitCode != 0) {
-      return "Commit " + instantTime + " failed to roll back";
-    }
-    return "Commit " + instantTime + " rolled back";
-  }
-
-  @CliCommand(value = "commit showpartitions", help = "Show partition level details of a commit")
+  @ShellMethod(key = "commit showpartitions", value = "Show partition level details of a commit")
   public String showCommitPartitions(
-      @CliOption(key = {"createView"}, mandatory = false, help = "view name to store output table",
-          unspecifiedDefaultValue = "") final String exportTableName,
-      @CliOption(key = {"commit"}, help = "Commit to show") final String instantTime,
-      @CliOption(key = {"limit"}, help = "Limit commits", unspecifiedDefaultValue = "-1") final Integer limit,
-      @CliOption(key = {"sortBy"}, help = "Sorting Field", unspecifiedDefaultValue = "") final String sortByField,
-      @CliOption(key = {"desc"}, help = "Ordering", unspecifiedDefaultValue = "false") final boolean descending,
-      @CliOption(key = {"headeronly"}, help = "Print Header Only",
-          unspecifiedDefaultValue = "false") final boolean headerOnly)
+      @ShellOption(value = {"--createView"}, help = "view name to store output table",
+          defaultValue = "") final String exportTableName,
+      @ShellOption(value = {"--commit"}, help = "Commit to show") final String instantTime,
+      @ShellOption(value = {"--limit"}, help = "Limit commits", defaultValue = "-1") final Integer limit,
+      @ShellOption(value = {"--sortBy"}, help = "Sorting Field", defaultValue = "") final String sortByField,
+      @ShellOption(value = {"--desc"}, help = "Ordering", defaultValue = "false") final boolean descending,
+      @ShellOption(value = {"--headeronly"}, help = "Print Header Only",
+          defaultValue = "false") final boolean headerOnly,
+      @ShellOption(value = {"includeArchivedTimeline"}, help = "Include archived commits as well",
+              defaultValue = "false") final boolean includeArchivedTimeline)
       throws Exception {
 
-    HoodieActiveTimeline activeTimeline = HoodieCLI.getTableMetaClient().getActiveTimeline();
-    HoodieTimeline timeline = activeTimeline.getCommitsTimeline().filterCompletedInstants();
+    HoodieDefaultTimeline defaultTimeline = getTimeline(HoodieCLI.getTableMetaClient(), includeArchivedTimeline);
+    HoodieTimeline timeline = defaultTimeline.getCommitsTimeline().filterCompletedInstants();
 
     Option<HoodieInstant> hoodieInstantOption = getCommitForInstant(timeline, instantTime);
     Option<HoodieCommitMetadata> commitMetadataOptional = getHoodieCommitMetadata(timeline, hoodieInstantOption);
@@ -323,20 +268,22 @@ public class CommitsCommand implements CommandMarker {
         limit, headerOnly, rows, exportTableName);
   }
 
-  @CliCommand(value = "commit show_write_stats", help = "Show write stats of a commit")
+  @ShellMethod(key = "commit show_write_stats", value = "Show write stats of a commit")
   public String showWriteStats(
-      @CliOption(key = {"createView"}, mandatory = false, help = "view name to store output table",
-          unspecifiedDefaultValue = "") final String exportTableName,
-      @CliOption(key = {"commit"}, help = "Commit to show") final String instantTime,
-      @CliOption(key = {"limit"}, help = "Limit commits", unspecifiedDefaultValue = "-1") final Integer limit,
-      @CliOption(key = {"sortBy"}, help = "Sorting Field", unspecifiedDefaultValue = "") final String sortByField,
-      @CliOption(key = {"desc"}, help = "Ordering", unspecifiedDefaultValue = "false") final boolean descending,
-      @CliOption(key = {"headeronly"}, help = "Print Header Only",
-          unspecifiedDefaultValue = "false") final boolean headerOnly)
+      @ShellOption(value = {"--createView"}, help = "view name to store output table",
+          defaultValue = "") final String exportTableName,
+      @ShellOption(value = {"--commit"}, help = "Commit to show") final String instantTime,
+      @ShellOption(value = {"--limit"}, help = "Limit commits", defaultValue = "-1") final Integer limit,
+      @ShellOption(value = {"--sortBy"}, help = "Sorting Field", defaultValue = "") final String sortByField,
+      @ShellOption(value = {"--desc"}, help = "Ordering", defaultValue = "false") final boolean descending,
+      @ShellOption(value = {"--headeronly"}, help = "Print Header Only",
+          defaultValue = "false") final boolean headerOnly,
+      @ShellOption(value = {"includeArchivedTimeline"}, help = "Include archived commits as well",
+              defaultValue = "false") final boolean includeArchivedTimeline)
       throws Exception {
 
-    HoodieActiveTimeline activeTimeline = HoodieCLI.getTableMetaClient().getActiveTimeline();
-    HoodieTimeline timeline = activeTimeline.getCommitsTimeline().filterCompletedInstants();
+    HoodieDefaultTimeline defaultTimeline = getTimeline(HoodieCLI.getTableMetaClient(), includeArchivedTimeline);
+    HoodieTimeline timeline = defaultTimeline.getCommitsTimeline().filterCompletedInstants();
 
     Option<HoodieInstant> hoodieInstantOption = getCommitForInstant(timeline, instantTime);
     Option<HoodieCommitMetadata> commitMetadataOptional = getHoodieCommitMetadata(timeline, hoodieInstantOption);
@@ -367,20 +314,22 @@ public class CommitsCommand implements CommandMarker {
         limit, headerOnly, rows, exportTableName);
   }
 
-  @CliCommand(value = "commit showfiles", help = "Show file level details of a commit")
+  @ShellMethod(key = "commit showfiles", value = "Show file level details of a commit")
   public String showCommitFiles(
-      @CliOption(key = {"createView"}, mandatory = false, help = "view name to store output table",
-          unspecifiedDefaultValue = "") final String exportTableName,
-      @CliOption(key = {"commit"}, help = "Commit to show") final String instantTime,
-      @CliOption(key = {"limit"}, help = "Limit commits", unspecifiedDefaultValue = "-1") final Integer limit,
-      @CliOption(key = {"sortBy"}, help = "Sorting Field", unspecifiedDefaultValue = "") final String sortByField,
-      @CliOption(key = {"desc"}, help = "Ordering", unspecifiedDefaultValue = "false") final boolean descending,
-      @CliOption(key = {"headeronly"}, help = "Print Header Only",
-          unspecifiedDefaultValue = "false") final boolean headerOnly)
+      @ShellOption(value = {"--createView"}, help = "view name to store output table",
+          defaultValue = "") final String exportTableName,
+      @ShellOption(value = {"--commit"}, help = "Commit to show") final String instantTime,
+      @ShellOption(value = {"--limit"}, help = "Limit commits", defaultValue = "-1") final Integer limit,
+      @ShellOption(value = {"--sortBy"}, help = "Sorting Field", defaultValue = "") final String sortByField,
+      @ShellOption(value = {"--desc"}, help = "Ordering", defaultValue = "false") final boolean descending,
+      @ShellOption(value = {"--headeronly"}, help = "Print Header Only",
+          defaultValue = "false") final boolean headerOnly,
+      @ShellOption(value = {"includeArchivedTimeline"}, help = "Include archived commits as well",
+              defaultValue = "false") final boolean includeArchivedTimeline)
       throws Exception {
 
-    HoodieActiveTimeline activeTimeline = HoodieCLI.getTableMetaClient().getActiveTimeline();
-    HoodieTimeline timeline = activeTimeline.getCommitsTimeline().filterCompletedInstants();
+    HoodieDefaultTimeline defaultTimeline = getTimeline(HoodieCLI.getTableMetaClient(), includeArchivedTimeline);
+    HoodieTimeline timeline = defaultTimeline.getCommitsTimeline().filterCompletedInstants();
 
     Option<HoodieInstant> hoodieInstantOption = getCommitForInstant(timeline, instantTime);
     Option<HoodieCommitMetadata> commitMetadataOptional = getHoodieCommitMetadata(timeline, hoodieInstantOption);
@@ -415,8 +364,8 @@ public class CommitsCommand implements CommandMarker {
         limit, headerOnly, rows, exportTableName);
   }
 
-  @CliCommand(value = "commits compare", help = "Compare commits with another Hoodie table")
-  public String compareCommits(@CliOption(key = {"path"}, help = "Path of the table to compare to") final String path) {
+  @ShellMethod(key = "commits compare", value = "Compare commits with another Hoodie table")
+  public String compareCommits(@ShellOption(value = {"--path"}, help = "Path of the table to compare to") final String path) {
 
     HoodieTableMetaClient source = HoodieCLI.getTableMetaClient();
     HoodieTableMetaClient target = HoodieTableMetaClient.builder().setConf(HoodieCLI.conf).setBasePath(path).build();
@@ -442,8 +391,8 @@ public class CommitsCommand implements CommandMarker {
     }
   }
 
-  @CliCommand(value = "commits sync", help = "Sync commits with another Hoodie table")
-  public String syncCommits(@CliOption(key = {"path"}, help = "Path of the table to sync to") final String path) {
+  @ShellMethod(key = "commits sync", value = "Sync commits with another Hoodie table")
+  public String syncCommits(@ShellOption(value = {"--path"}, help = "Path of the table to sync to") final String path) {
     HoodieCLI.syncTableMetadata = HoodieTableMetaClient.builder().setConf(HoodieCLI.conf).setBasePath(path).build();
     HoodieCLI.state = HoodieCLI.CLIState.SYNC;
     return "Load sync state between " + HoodieCLI.getTableMetaClient().getTableConfig().getTableName() + " and "
@@ -453,15 +402,13 @@ public class CommitsCommand implements CommandMarker {
   /*
   Checks whether a commit or replacecommit action exists in the timeline.
   * */
-  private Option<HoodieInstant> getCommitForInstant(HoodieTimeline timeline, String instantTime) throws IOException {
+  private Option<HoodieInstant> getCommitForInstant(HoodieTimeline timeline, String instantTime) {
     List<HoodieInstant> instants = Arrays.asList(
-            new HoodieInstant(false, HoodieTimeline.COMMIT_ACTION, instantTime),
-            new HoodieInstant(false, HoodieTimeline.REPLACE_COMMIT_ACTION, instantTime),
-            new HoodieInstant(false, HoodieTimeline.DELTA_COMMIT_ACTION, instantTime));
+        new HoodieInstant(false, HoodieTimeline.COMMIT_ACTION, instantTime),
+        new HoodieInstant(false, HoodieTimeline.REPLACE_COMMIT_ACTION, instantTime),
+        new HoodieInstant(false, HoodieTimeline.DELTA_COMMIT_ACTION, instantTime));
 
-    Option<HoodieInstant> hoodieInstant = Option.fromJavaOptional(instants.stream().filter(timeline::containsInstant).findAny());
-
-    return hoodieInstant;
+    return Option.fromJavaOptional(instants.stream().filter(timeline::containsInstant).findAny());
   }
 
   private Option<HoodieCommitMetadata> getHoodieCommitMetadata(HoodieTimeline timeline, Option<HoodieInstant> hoodieInstant) throws IOException {
@@ -471,7 +418,7 @@ public class CommitsCommand implements CommandMarker {
             HoodieReplaceCommitMetadata.class));
       }
       return Option.of(HoodieCommitMetadata.fromBytes(timeline.getInstantDetails(hoodieInstant.get()).get(),
-              HoodieCommitMetadata.class));
+          HoodieCommitMetadata.class));
     }
 
     return Option.empty();
