@@ -28,10 +28,13 @@ import org.apache.hudi.common.config.TypedProperties;
 import org.apache.hudi.common.engine.HoodieEngineContext;
 import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.model.HoodieFailedWritesCleaningPolicy;
+import org.apache.hudi.common.model.HoodieRecord;
+import org.apache.hudi.common.model.WriteOperationType;
 import org.apache.hudi.common.table.HoodieTableMetaClient;
 import org.apache.hudi.common.table.HoodieTableVersion;
 import org.apache.hudi.common.table.timeline.versioning.TimelineLayoutVersion;
 import org.apache.hudi.common.util.Option;
+import org.apache.hudi.common.util.PartitionPathEncodeUtils;
 import org.apache.hudi.common.util.StringUtils;
 import org.apache.hudi.common.util.ValidationUtils;
 import org.apache.hudi.config.HoodieBootstrapConfig;
@@ -56,16 +59,24 @@ import org.apache.hudi.utilities.HoodieCompactor;
 import org.apache.hudi.utilities.deltastreamer.BootstrapExecutor;
 import org.apache.hudi.utilities.deltastreamer.HoodieDeltaStreamer;
 
+import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
-import org.apache.log4j.Logger;
+import org.apache.logging.log4j.LogManager;
+import org.apache.logging.log4j.Logger;
 import org.apache.spark.api.java.JavaSparkContext;
+import org.apache.spark.sql.Dataset;
+import org.apache.spark.sql.Row;
 import org.apache.spark.sql.SQLContext;
+import org.apache.spark.sql.functions;
+import org.apache.spark.sql.types.StructType;
 
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Locale;
+import java.util.Map;
 
 import static org.apache.hudi.utilities.UtilHelpers.EXECUTE;
 import static org.apache.hudi.utilities.UtilHelpers.SCHEDULE;
@@ -78,7 +89,7 @@ import static org.apache.hudi.utilities.UtilHelpers.readConfig;
  */
 public class SparkMain {
 
-  private static final Logger LOG = Logger.getLogger(SparkMain.class);
+  private static final Logger LOG = LogManager.getLogger(SparkMain.class);
 
   /**
    * Commands.
@@ -86,7 +97,8 @@ public class SparkMain {
   enum SparkCommand {
     BOOTSTRAP, ROLLBACK, DEDUPLICATE, ROLLBACK_TO_SAVEPOINT, SAVEPOINT, IMPORT, UPSERT, COMPACT_SCHEDULE, COMPACT_RUN, COMPACT_SCHEDULE_AND_EXECUTE,
     COMPACT_UNSCHEDULE_PLAN, COMPACT_UNSCHEDULE_FILE, COMPACT_VALIDATE, COMPACT_REPAIR, CLUSTERING_SCHEDULE,
-    CLUSTERING_RUN, CLUSTERING_SCHEDULE_AND_EXECUTE, CLEAN, DELETE_MARKER, DELETE_SAVEPOINT, UPGRADE, DOWNGRADE
+    CLUSTERING_RUN, CLUSTERING_SCHEDULE_AND_EXECUTE, CLEAN, DELETE_MARKER, DELETE_SAVEPOINT, UPGRADE, DOWNGRADE,
+    REPAIR_DEPRECATED_PARTITION, RENAME_PARTITION
   }
 
   public static void main(String[] args) throws Exception {
@@ -174,7 +186,7 @@ public class SparkMain {
         case COMPACT_REPAIR:
           assert (args.length == 8);
           doCompactRepair(jsc, args[3], args[4], args[5], Integer.parseInt(args[6]),
-                  Boolean.parseBoolean(args[7]));
+              Boolean.parseBoolean(args[7]));
           returnCode = 0;
           break;
         case COMPACT_UNSCHEDULE_FILE:
@@ -263,13 +275,21 @@ public class SparkMain {
             configs.addAll(Arrays.asList(args).subList(18, args.length));
           }
           returnCode = doBootstrap(jsc, args[3], args[4], args[5], args[6], args[7], args[8], args[9], args[10],
-                  args[11], args[12], args[13], args[14], args[15], args[16], propsFilePath, configs);
+              args[11], args[12], args[13], args[14], args[15], args[16], propsFilePath, configs);
           break;
         case UPGRADE:
         case DOWNGRADE:
           assert (args.length == 5);
           returnCode = upgradeOrDowngradeTable(jsc, args[3], args[4]);
           break;
+        case REPAIR_DEPRECATED_PARTITION:
+          assert (args.length == 4);
+          returnCode = repairDeprecatedPartition(jsc, args[3]);
+          break;
+        case RENAME_PARTITION:
+          assert (args.length == 6);
+          returnCode = renamePartition(jsc, args[3], args[4], args[5]);
+          break;
         default:
           break;
       }
@@ -283,7 +303,7 @@ public class SparkMain {
   }
 
   protected static void clean(JavaSparkContext jsc, String basePath, String propsFilePath,
-      List<String> configs) {
+                              List<String> configs) {
     HoodieCleaner.Config cfg = new HoodieCleaner.Config();
     cfg.basePath = basePath;
     cfg.propsFilePath = propsFilePath;
@@ -307,8 +327,8 @@ public class SparkMain {
   }
 
   private static int dataLoad(JavaSparkContext jsc, String command, String srcPath, String targetPath, String tableName,
-      String tableType, String rowKey, String partitionKey, int parallelism, String schemaFile,
-      int retry, String propsFilePath, List<String> configs) {
+                              String tableType, String rowKey, String partitionKey, int parallelism, String schemaFile,
+                              int retry, String propsFilePath, List<String> configs) {
     Config cfg = new Config();
     cfg.command = command;
     cfg.srcPath = srcPath;
@@ -325,7 +345,7 @@ public class SparkMain {
   }
 
   private static void doCompactValidate(JavaSparkContext jsc, String basePath, String compactionInstant,
-      String outputPath, int parallelism) throws Exception {
+                                        String outputPath, int parallelism) throws Exception {
     HoodieCompactionAdminTool.Config cfg = new HoodieCompactionAdminTool.Config();
     cfg.basePath = basePath;
     cfg.operation = Operation.VALIDATE;
@@ -336,7 +356,7 @@ public class SparkMain {
   }
 
   private static void doCompactRepair(JavaSparkContext jsc, String basePath, String compactionInstant,
-      String outputPath, int parallelism, boolean dryRun) throws Exception {
+                                      String outputPath, int parallelism, boolean dryRun) throws Exception {
     HoodieCompactionAdminTool.Config cfg = new HoodieCompactionAdminTool.Config();
     cfg.basePath = basePath;
     cfg.operation = Operation.REPAIR;
@@ -348,7 +368,7 @@ public class SparkMain {
   }
 
   private static void doCompactUnschedule(JavaSparkContext jsc, String basePath, String compactionInstant,
-      String outputPath, int parallelism, boolean skipValidation, boolean dryRun) throws Exception {
+                                          String outputPath, int parallelism, boolean skipValidation, boolean dryRun) throws Exception {
     HoodieCompactionAdminTool.Config cfg = new HoodieCompactionAdminTool.Config();
     cfg.basePath = basePath;
     cfg.operation = Operation.UNSCHEDULE_PLAN;
@@ -361,7 +381,7 @@ public class SparkMain {
   }
 
   private static void doCompactUnscheduleFile(JavaSparkContext jsc, String basePath, String fileId, String partitionPath,
-      String outputPath, int parallelism, boolean skipValidation, boolean dryRun)
+                                              String outputPath, int parallelism, boolean skipValidation, boolean dryRun)
       throws Exception {
     HoodieCompactionAdminTool.Config cfg = new HoodieCompactionAdminTool.Config();
     cfg.basePath = basePath;
@@ -376,8 +396,8 @@ public class SparkMain {
   }
 
   private static int compact(JavaSparkContext jsc, String basePath, String tableName, String compactionInstant,
-      int parallelism, String schemaFile, int retry, String mode, String propsFilePath,
-      List<String> configs) {
+                             int parallelism, String schemaFile, int retry, String mode, String propsFilePath,
+                             List<String> configs) {
     HoodieCompactor.Config cfg = new HoodieCompactor.Config();
     cfg.basePath = basePath;
     cfg.tableName = tableName;
@@ -393,7 +413,7 @@ public class SparkMain {
   }
 
   private static int cluster(JavaSparkContext jsc, String basePath, String tableName, String clusteringInstant,
-      int parallelism, String sparkMemory, int retry, String runningMode, String propsFilePath, List<String> configs) {
+                             int parallelism, String sparkMemory, int retry, String runningMode, String propsFilePath, List<String> configs) {
     HoodieClusteringJob.Config cfg = new HoodieClusteringJob.Config();
     cfg.basePath = basePath;
     cfg.tableName = tableName;
@@ -407,17 +427,99 @@ public class SparkMain {
   }
 
   private static int deduplicatePartitionPath(JavaSparkContext jsc, String duplicatedPartitionPath,
-      String repairedOutputPath, String basePath, boolean dryRun, String dedupeType) {
+                                              String repairedOutputPath, String basePath, boolean dryRun, String dedupeType) {
     DedupeSparkJob job = new DedupeSparkJob(basePath, duplicatedPartitionPath, repairedOutputPath, new SQLContext(jsc),
         FSUtils.getFs(basePath, jsc.hadoopConfiguration()), DeDupeType.withName(dedupeType));
     job.fixDuplicates(dryRun);
     return 0;
   }
 
+  public static int repairDeprecatedPartition(JavaSparkContext jsc, String basePath) {
+    SQLContext sqlContext = new SQLContext(jsc);
+    Dataset<Row> recordsToRewrite = getRecordsToRewrite(basePath, PartitionPathEncodeUtils.DEPRECATED_DEFAULT_PARTITION_PATH, sqlContext);
+
+    if (!recordsToRewrite.isEmpty()) {
+      recordsToRewrite.cache();
+      HoodieTableMetaClient metaClient = HoodieTableMetaClient.builder().setConf(jsc.hadoopConfiguration()).setBasePath(basePath).build();
+      Map<String, String> propsMap = getPropsForRewrite(metaClient);
+      rewriteRecordsToNewPartition(basePath, PartitionPathEncodeUtils.DEFAULT_PARTITION_PATH, recordsToRewrite, metaClient, propsMap);
+      // after re-writing, we can safely delete older data.
+      deleteOlderPartition(basePath, PartitionPathEncodeUtils.DEPRECATED_DEFAULT_PARTITION_PATH, recordsToRewrite, propsMap);
+    }
+    return 0;
+  }
+
+  public static int renamePartition(JavaSparkContext jsc, String basePath, String oldPartition, String newPartition) {
+    SQLContext sqlContext = new SQLContext(jsc);
+    Dataset<Row> recordsToRewrite = getRecordsToRewrite(basePath, oldPartition, sqlContext);
+
+    if (!recordsToRewrite.isEmpty()) {
+      recordsToRewrite.cache();
+      HoodieTableMetaClient metaClient = HoodieTableMetaClient.builder().setConf(jsc.hadoopConfiguration()).setBasePath(basePath).build();
+      Map<String, String> propsMap = getPropsForRewrite(metaClient);
+      rewriteRecordsToNewPartition(basePath, newPartition, recordsToRewrite, metaClient, propsMap);
+      // after re-writing, we can safely delete older partition.
+      deleteOlderPartition(basePath, oldPartition, recordsToRewrite, propsMap);
+      // also, we can physically delete the old partition.
+      FileSystem fs = FSUtils.getFs(new Path(basePath), metaClient.getHadoopConf());
+      try {
+        fs.delete(new Path(basePath, oldPartition), true);
+      } catch (IOException e) {
+        LOG.warn("Failed to delete older partition " + basePath);
+      }
+    }
+    return 0;
+  }
+
+  private static void deleteOlderPartition(String basePath, String oldPartition, Dataset<Row> recordsToRewrite, Map<String, String> propsMap) {
+    propsMap.put("hoodie.datasource.write.partitions.to.delete", oldPartition);
+    recordsToRewrite.write()
+        .options(propsMap)
+        .option("hoodie.datasource.write.operation", WriteOperationType.DELETE_PARTITION.value())
+        .format("hudi")
+        .mode("Append")
+        .save(basePath);
+  }
+
+  private static void rewriteRecordsToNewPartition(String basePath, String newPartition, Dataset<Row> recordsToRewrite, HoodieTableMetaClient metaClient, Map<String, String> propsMap) {
+    String partitionFieldProp = metaClient.getTableConfig().getPartitionFieldProp();
+    StructType structType = recordsToRewrite.schema();
+    int partitionIndex = structType.fieldIndex(partitionFieldProp);
+
+    recordsToRewrite.withColumn(metaClient.getTableConfig().getPartitionFieldProp(), functions.lit(null).cast(structType.apply(partitionIndex).dataType()))
+        .write()
+        .options(propsMap)
+        .option("hoodie.datasource.write.operation", WriteOperationType.BULK_INSERT.value())
+        .format("hudi")
+        .mode("Append")
+        .save(basePath);
+  }
+
+  private static Dataset<Row> getRecordsToRewrite(String basePath, String oldPartition, SQLContext sqlContext) {
+    return sqlContext.read()
+        .format("hudi")
+        .load(basePath + "/" + oldPartition)
+        .drop(HoodieRecord.RECORD_KEY_METADATA_FIELD)
+        .drop(HoodieRecord.PARTITION_PATH_METADATA_FIELD)
+        .drop(HoodieRecord.COMMIT_SEQNO_METADATA_FIELD)
+        .drop(HoodieRecord.FILENAME_METADATA_FIELD)
+        .drop(HoodieRecord.COMMIT_TIME_METADATA_FIELD);
+  }
+
+  private static Map<String, String> getPropsForRewrite(HoodieTableMetaClient metaClient) {
+    Map<String, String> propsMap = new HashMap<>();
+    metaClient.getTableConfig().getProps().forEach((k, v) -> propsMap.put(k.toString(), v.toString()));
+    propsMap.put(HoodieWriteConfig.SKIP_DEFAULT_PARTITION_VALIDATION.key(), "true");
+    propsMap.put(DataSourceWriteOptions.RECORDKEY_FIELD().key(), metaClient.getTableConfig().getRecordKeyFieldProp());
+    propsMap.put(DataSourceWriteOptions.PARTITIONPATH_FIELD().key(), metaClient.getTableConfig().getPartitionFieldProp());
+    propsMap.put(DataSourceWriteOptions.KEYGENERATOR_CLASS_NAME().key(), metaClient.getTableConfig().getKeyGeneratorClassName());
+    return propsMap;
+  }
+
   private static int doBootstrap(JavaSparkContext jsc, String tableName, String tableType, String basePath,
-      String sourcePath, String recordKeyCols, String partitionFields, String parallelism, String schemaProviderClass,
-      String bootstrapIndexClass, String selectorClass, String keyGenerator, String fullBootstrapInputProvider,
-      String payloadClassName, String enableHiveSync, String propsFilePath, List<String> configs) throws IOException {
+                                 String sourcePath, String recordKeyCols, String partitionFields, String parallelism, String schemaProviderClass,
+                                 String bootstrapIndexClass, String selectorClass, String keyGenerator, String fullBootstrapInputProvider,
+                                 String payloadClassName, String enableHiveSync, String propsFilePath, List<String> configs) throws IOException {
 
     TypedProperties properties = propsFilePath == null ? buildProperties(configs)
         : readConfig(jsc.hadoopConfiguration(), new Path(propsFilePath), configs).getProps(true);
@@ -462,7 +564,7 @@ public class SparkMain {
   }
 
   private static int createSavepoint(JavaSparkContext jsc, String commitTime, String user,
-      String comments, String basePath) throws Exception {
+                                     String comments, String basePath) throws Exception {
     SparkRDDWriteClient client = createHoodieClient(jsc, basePath, false);
     try {
       client.savepoint(commitTime, user, comments);
@@ -501,8 +603,8 @@ public class SparkMain {
   /**
    * Upgrade or downgrade table.
    *
-   * @param jsc instance of {@link JavaSparkContext} to use.
-   * @param basePath base path of the dataset.
+   * @param jsc       instance of {@link JavaSparkContext} to use.
+   * @param basePath  base path of the dataset.
    * @param toVersion version to which upgrade/downgrade to be done.
    * @return 0 if success, else -1.
    * @throws Exception
@@ -515,8 +617,10 @@ public class SparkMain {
             .setLoadActiveTimelineOnLoad(false).setConsistencyGuardConfig(config.getConsistencyGuardConfig())
             .setLayoutVersion(Option.of(new TimelineLayoutVersion(config.getTimelineLayoutVersion())))
             .setFileSystemRetryConfig(config.getFileSystemRetryConfig()).build();
+    HoodieWriteConfig updatedConfig = HoodieWriteConfig.newBuilder().withProps(config.getProps())
+        .forTable(metaClient.getTableConfig().getTableName()).build();
     try {
-      new UpgradeDowngrade(metaClient, config, new HoodieSparkEngineContext(jsc), SparkUpgradeDowngradeHelper.getInstance())
+      new UpgradeDowngrade(metaClient, updatedConfig, new HoodieSparkEngineContext(jsc), SparkUpgradeDowngradeHelper.getInstance())
           .run(HoodieTableVersion.valueOf(toVersion), null);
       LOG.info(String.format("Table at \"%s\" upgraded / downgraded to version \"%s\".", basePath, toVersion));
       return 0;
