@@ -25,6 +25,7 @@ import org.apache.hudi.cli.HoodieTableHeaderFields;
 import org.apache.hudi.cli.TableHeader;
 import org.apache.hudi.cli.functional.CLIFunctionalTestHarness;
 import org.apache.hudi.cli.testutils.HoodieTestCommitMetadataGenerator;
+import org.apache.hudi.cli.testutils.ShellEvaluationResultUtil;
 import org.apache.hudi.common.config.HoodieCommonConfig;
 import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.model.HoodieLogFile;
@@ -51,7 +52,9 @@ import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Tag;
 import org.junit.jupiter.api.Test;
-import org.springframework.shell.core.CommandResult;
+import org.springframework.beans.factory.annotation.Autowired;
+import org.springframework.boot.test.context.SpringBootTest;
+import org.springframework.shell.Shell;
 
 import java.io.IOException;
 import java.net.URISyntaxException;
@@ -74,8 +77,12 @@ import static org.junit.jupiter.api.Assertions.assertTrue;
  * Test Cases for {@link HoodieLogFileCommand}.
  */
 @Tag("functional")
+@SpringBootTest(properties = {"spring.shell.interactive.enabled=false", "spring.shell.command.script.enabled=false"})
 public class TestHoodieLogFileCommand extends CLIFunctionalTestHarness {
 
+  @Autowired
+  private Shell shell;
+
   private String partitionPath;
   private HoodieAvroDataBlock dataBlock;
   private String tablePath;
@@ -123,8 +130,8 @@ public class TestHoodieLogFileCommand extends CLIFunctionalTestHarness {
    */
   @Test
   public void testShowLogFileCommits() throws JsonProcessingException {
-    CommandResult cr = shell().executeCommand("show logfile metadata --logFilePathPattern " + partitionPath + "/*");
-    assertTrue(cr.isSuccess());
+    Object result = shell.evaluate(() -> "show logfile metadata --logFilePathPattern " + partitionPath + "/*");
+    assertTrue(ShellEvaluationResultUtil.isSuccess(result));
 
     TableHeader header = new TableHeader().addTableHeaderField(HoodieTableHeaderFields.HEADER_INSTANT_TIME)
         .addTableHeaderField(HoodieTableHeaderFields.HEADER_RECORD_COUNT)
@@ -137,12 +144,12 @@ public class TestHoodieLogFileCommand extends CLIFunctionalTestHarness {
     ObjectMapper objectMapper = new ObjectMapper();
     String headerStr = objectMapper.writeValueAsString(dataBlock.getLogBlockHeader());
     String footerStr = objectMapper.writeValueAsString(dataBlock.getLogBlockFooter());
-    Comparable[] output = new Comparable[]{INSTANT_TIME, 100, dataBlock.getBlockType(), headerStr, footerStr};
+    Comparable[] output = new Comparable[] {INSTANT_TIME, 100, dataBlock.getBlockType(), headerStr, footerStr};
     rows.add(output);
 
     String expected = HoodiePrintHelper.print(header, new HashMap<>(), "", false, -1, false, rows);
     expected = removeNonWordAndStripSpace(expected);
-    String got = removeNonWordAndStripSpace(cr.getResult().toString());
+    String got = removeNonWordAndStripSpace(result.toString());
     assertEquals(expected, got);
   }
 
@@ -151,15 +158,15 @@ public class TestHoodieLogFileCommand extends CLIFunctionalTestHarness {
    */
   @Test
   public void testShowLogFileRecords() throws IOException, URISyntaxException {
-    CommandResult cr = shell().executeCommand("show logfile records --logFilePathPattern " + partitionPath + "/*");
-    assertTrue(cr.isSuccess());
+    Object result = shell.evaluate(() -> "show logfile records --logFilePathPattern " + partitionPath + "/*");
+    assertTrue(ShellEvaluationResultUtil.isSuccess(result));
 
     // construct expect result, get 10 records.
     List<IndexedRecord> records = SchemaTestUtil.generateTestRecords(0, 10);
-    String[][] rows = records.stream().map(r -> new String[]{r.toString()}).toArray(String[][]::new);
+    String[][] rows = records.stream().map(r -> new String[] {r.toString()}).toArray(String[][]::new);
     String expected = HoodiePrintHelper.print(new String[] {HoodieTableHeaderFields.HEADER_RECORDS}, rows);
     expected = removeNonWordAndStripSpace(expected);
-    String got = removeNonWordAndStripSpace(cr.getResult().toString());
+    String got = removeNonWordAndStripSpace(result.toString());
     assertEquals(expected, got);
   }
 
@@ -196,9 +203,9 @@ public class TestHoodieLogFileCommand extends CLIFunctionalTestHarness {
       }
     }
 
-    CommandResult cr = shell().executeCommand("show logfile records --logFilePathPattern "
-        + partitionPath + "/* --mergeRecords true");
-    assertTrue(cr.isSuccess());
+    Object result = shell.evaluate(() -> "show logfile records --logFilePathPattern "
+            + partitionPath + "/* --mergeRecords true");
+    assertTrue(ShellEvaluationResultUtil.isSuccess(result));
 
     // get expected result of 10 records.
     List<String> logFilePaths = Arrays.stream(fs.globStatus(new Path(partitionPath + "/*")))
@@ -232,12 +239,12 @@ public class TestHoodieLogFileCommand extends CLIFunctionalTestHarness {
       indexRecords.add(hoodieRecord.get());
       num++;
     }
-    String[][] rows = indexRecords.stream().map(r -> new String[]{r.toString()}).toArray(String[][]::new);
+    String[][] rows = indexRecords.stream().map(r -> new String[] {r.toString()}).toArray(String[][]::new);
     assertNotNull(rows);
 
     String expected = HoodiePrintHelper.print(new String[] {HoodieTableHeaderFields.HEADER_RECORDS}, rows);
     expected = removeNonWordAndStripSpace(expected);
-    String got = removeNonWordAndStripSpace(cr.getResult().toString());
+    String got = removeNonWordAndStripSpace(result.toString());
     assertEquals(expected, got);
   }
 }
