@@ -21,9 +21,12 @@ package org.apache.hudi.cli.commands;
 import org.apache.hudi.cli.HoodiePrintHelper;
 import org.apache.hudi.cli.functional.CLIFunctionalTestHarness;
 
+import org.apache.hudi.cli.testutils.ShellEvaluationResultUtil;
 import org.junit.jupiter.api.Tag;
 import org.junit.jupiter.api.Test;
-import org.springframework.shell.core.CommandResult;
+import org.springframework.beans.factory.annotation.Autowired;
+import org.springframework.boot.test.context.SpringBootTest;
+import org.springframework.shell.Shell;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertTrue;
@@ -32,29 +35,33 @@ import static org.junit.jupiter.api.Assertions.assertTrue;
  * Test Cases for {@link SparkEnvCommand}.
  */
 @Tag("functional")
+@SpringBootTest(properties = {"spring.shell.interactive.enabled=false", "spring.shell.command.script.enabled=false"})
 public class TestSparkEnvCommand extends CLIFunctionalTestHarness {
 
+  @Autowired
+  private Shell shell;
+
   /**
    * Test Cases for set and get spark env.
    */
   @Test
   public void testSetAndGetSparkEnv() {
     // First, be empty
-    CommandResult cr = shell().executeCommand("show envs all");
+    Object cmdResult = shell.evaluate(() -> "show envs all");
     String nullResult = HoodiePrintHelper.print(new String[] {"key", "value"}, new String[0][2]);
     nullResult = removeNonWordAndStripSpace(nullResult);
-    String got = removeNonWordAndStripSpace(cr.getResult().toString());
+    String got = removeNonWordAndStripSpace(cmdResult.toString());
     assertEquals(nullResult, got);
 
     // Set SPARK_HOME
-    cr = shell().executeCommand("set --conf SPARK_HOME=/usr/etc/spark");
-    assertTrue(cr.isSuccess());
+    cmdResult = shell.evaluate(() -> "set --conf SPARK_HOME=/usr/etc/spark");
+    assertTrue(ShellEvaluationResultUtil.isSuccess(cmdResult));
 
     //Get
-    cr = shell().executeCommand("show env --key SPARK_HOME");
+    cmdResult = shell.evaluate(() -> "show env --key SPARK_HOME");
     String result = HoodiePrintHelper.print(new String[] {"key", "value"}, new String[][] {new String[] {"SPARK_HOME", "/usr/etc/spark"}});
     result = removeNonWordAndStripSpace(result);
-    got = removeNonWordAndStripSpace(cr.getResult().toString());
+    got = removeNonWordAndStripSpace(cmdResult.toString());
     assertEquals(result, got);
   }
 }
