@@ -22,6 +22,7 @@ import org.apache.hadoop.fs.Path;
 import org.apache.hudi.cli.HoodieCLI;
 import org.apache.hudi.cli.commands.TableCommand;
 import org.apache.hudi.cli.testutils.HoodieCLIIntegrationTestBase;
+import org.apache.hudi.cli.testutils.ShellEvaluationResultUtil;
 import org.apache.hudi.client.common.HoodieSparkEngineContext;
 import org.apache.hudi.common.config.HoodieMetadataConfig;
 import org.apache.hudi.common.model.HoodieTableType;
@@ -36,7 +37,9 @@ import org.apache.hudi.metadata.HoodieTableMetadata;
 import org.apache.hudi.metadata.SparkHoodieBackedTableMetadataWriter;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
-import org.springframework.shell.core.CommandResult;
+import org.springframework.beans.factory.annotation.Autowired;
+import org.springframework.boot.test.context.SpringBootTest;
+import org.springframework.shell.Shell;
 
 import java.io.IOException;
 
@@ -51,8 +54,11 @@ import static org.junit.jupiter.api.Assertions.assertTrue;
  * A command use SparkLauncher need load jars under lib which generate during mvn package.
  * Use integration test instead of unit test.
  */
+@SpringBootTest(properties = {"spring.shell.interactive.enabled=false", "spring.shell.command.script.enabled=false"})
 public class ITTestSavepointsCommand extends HoodieCLIIntegrationTestBase {
 
+  @Autowired
+  private Shell shell;
   private String tablePath;
 
   @BeforeEach
@@ -78,13 +84,13 @@ public class ITTestSavepointsCommand extends HoodieCLIIntegrationTestBase {
     }
 
     String savepoint = "102";
-    CommandResult cr = getShell().executeCommand(
-        String.format("savepoint create --commit %s --sparkMaster %s", savepoint, "local"));
+    Object result = shell.evaluate(() ->
+            String.format("savepoint create --commit %s --sparkMaster %s", savepoint, "local"));
 
     assertAll("Command run failed",
-        () -> assertTrue(cr.isSuccess()),
+        () -> assertTrue(ShellEvaluationResultUtil.isSuccess(result)),
         () -> assertEquals(
-            String.format("The commit \"%s\" has been savepointed.", savepoint), cr.getResult().toString()));
+            String.format("The commit \"%s\" has been savepointed.", savepoint), result.toString()));
 
     // there is 1 savepoint instant
     HoodieActiveTimeline timeline = HoodieCLI.getTableMetaClient().getActiveTimeline();
@@ -106,13 +112,13 @@ public class ITTestSavepointsCommand extends HoodieCLIIntegrationTestBase {
     String savepoint = "102";
     HoodieTestDataGenerator.createSavepointFile(tablePath, savepoint, jsc.hadoopConfiguration());
 
-    CommandResult cr = getShell().executeCommand(
-        String.format("savepoint rollback --savepoint %s --sparkMaster %s", savepoint, "local"));
+    Object result = shell.evaluate(() ->
+            String.format("savepoint rollback --savepoint %s --sparkMaster %s", savepoint, "local"));
 
     assertAll("Command run failed",
-        () -> assertTrue(cr.isSuccess()),
+        () -> assertTrue(ShellEvaluationResultUtil.isSuccess(result)),
         () -> assertEquals(
-            String.format("Savepoint \"%s\" rolled back", savepoint), cr.getResult().toString()));
+            String.format("Savepoint \"%s\" rolled back", savepoint), result.toString()));
 
     // there is 1 restore instant
     HoodieActiveTimeline timeline = HoodieCLI.getTableMetaClient().getActiveTimeline();
@@ -148,13 +154,13 @@ public class ITTestSavepointsCommand extends HoodieCLIIntegrationTestBase {
     assertTrue(HoodieCLI.fs.exists(metadataTableBasePath));
 
     // roll back to savepoint
-    CommandResult cr = getShell().executeCommand(
-        String.format("savepoint rollback --savepoint %s --sparkMaster %s", savepoint, "local"));
+    Object result = shell.evaluate(() ->
+            String.format("savepoint rollback --savepoint %s --sparkMaster %s", savepoint, "local"));
 
     assertAll("Command run failed",
-        () -> assertTrue(cr.isSuccess()),
+        () -> assertTrue(ShellEvaluationResultUtil.isSuccess(result)),
         () -> assertEquals(
-            String.format("Savepoint \"%s\" rolled back", savepoint), cr.getResult().toString()));
+            String.format("Savepoint \"%s\" rolled back", savepoint), result.toString()));
 
     // there is 1 restore instant
     HoodieActiveTimeline timeline = HoodieCLI.getTableMetaClient().getActiveTimeline();
@@ -187,13 +193,13 @@ public class ITTestSavepointsCommand extends HoodieCLIIntegrationTestBase {
     HoodieActiveTimeline timeline = HoodieCLI.getTableMetaClient().getActiveTimeline();
     assertEquals(2, timeline.getSavePointTimeline().countInstants(), "There should 2 instants.");
 
-    CommandResult cr = getShell().executeCommand(
-        String.format("savepoint delete --commit %s --sparkMaster %s", savepoint1, "local"));
+    Object result = shell.evaluate(() ->
+            String.format("savepoint delete --commit %s --sparkMaster %s", savepoint1, "local"));
 
     assertAll("Command run failed",
-        () -> assertTrue(cr.isSuccess()),
+        () -> assertTrue(ShellEvaluationResultUtil.isSuccess(result)),
         () -> assertEquals(
-            String.format("Savepoint \"%s\" deleted.", savepoint1), cr.getResult().toString()));
+            String.format("Savepoint \"%s\" deleted.", savepoint1),result.toString()));
 
     // reload timeline
     timeline = timeline.reload();
