@@ -18,8 +18,6 @@
 
 package org.apache.hudi.metrics;
 
-import java.io.Closeable;
-
 /**
  * Used for testing.
  */
@@ -31,11 +29,6 @@ public class InMemoryMetricsReporter extends MetricsReporter {
   @Override
   public void report() {}
 
-  @Override
-  public Closeable getReporter() {
-    return null;
-  }
-
   @Override
   public void stop() {
 
