@@ -74,6 +74,11 @@ public class FlinkAppendHandle<T extends HoodieRecordPayload, I, K, O>
     writeMarkers.createIfNotExists(partitionPath, dataFileName, getIOType());
   }
 
+  @Override
+  public boolean canWrite(HoodieRecord record) {
+    return true;
+  }
+
   @Override
   protected boolean needsUpdateLocation() {
     return false;
