@@ -18,14 +18,14 @@
 
 package org.apache.hudi.io.storage.row;
 
+import org.apache.hadoop.fs.Path;
 import org.apache.hudi.common.bloom.BloomFilter;
 import org.apache.hudi.common.bloom.BloomFilterFactory;
 import org.apache.hudi.common.fs.FSUtils;
+import org.apache.hudi.common.util.Option;
 import org.apache.hudi.config.HoodieWriteConfig;
-import org.apache.hudi.exception.HoodieIOException;
+import org.apache.hudi.io.storage.HoodieParquetConfig;
 import org.apache.hudi.table.HoodieTable;
-
-import org.apache.hadoop.fs.Path;
 import org.apache.spark.sql.types.StructType;
 
 import java.io.IOException;
@@ -41,65 +41,60 @@ public class HoodieInternalRowFileWriterFactory {
    * Factory method to assist in instantiating an instance of {@link HoodieInternalRowFileWriter}.
    * @param path path of the RowFileWriter.
    * @param hoodieTable instance of {@link HoodieTable} in use.
-   * @param config instance of {@link HoodieWriteConfig} to use.
+   * @param writeConfig instance of {@link HoodieWriteConfig} to use.
    * @param schema schema of the dataset in use.
    * @return the instantiated {@link HoodieInternalRowFileWriter}.
    * @throws IOException if format is not supported or if any exception during instantiating the RowFileWriter.
    *
    */
-  public static HoodieInternalRowFileWriter getInternalRowFileWriter(
-      Path path, HoodieTable hoodieTable, HoodieWriteConfig config, StructType schema)
+  public static HoodieInternalRowFileWriter getInternalRowFileWriter(Path path,
+                                                                     HoodieTable hoodieTable,
+                                                                     HoodieWriteConfig writeConfig,
+                                                                     StructType schema)
       throws IOException {
     final String extension = FSUtils.getFileExtension(path.getName());
     if (PARQUET.getFileExtension().equals(extension)) {
-      return newParquetInternalRowFileWriter(path, config, schema, hoodieTable);
+      return newParquetInternalRowFileWriter(path, hoodieTable, writeConfig, schema, tryInstantiateBloomFilter(writeConfig));
     }
     throw new UnsupportedOperationException(extension + " format not supported yet.");
   }
 
-  private static HoodieInternalRowFileWriter newParquetInternalRowFileWriter(
-      Path path, HoodieWriteConfig writeConfig, StructType structType, HoodieTable table)
+  private static HoodieInternalRowFileWriter newParquetInternalRowFileWriter(Path path,
+                                                                             HoodieTable table,
+                                                                             HoodieWriteConfig writeConfig,
+                                                                             StructType structType,
+                                                                             Option<BloomFilter> bloomFilterOpt
+  )
       throws IOException {
-    BloomFilter filter = BloomFilterFactory.createBloomFilter(
-            writeConfig.getBloomFilterNumEntries(),
-            writeConfig.getBloomFilterFPP(),
-            writeConfig.getDynamicBloomFilterMaxNumEntries(),
-            writeConfig.getBloomFilterType());
     HoodieRowParquetWriteSupport writeSupport =
-            new HoodieRowParquetWriteSupport(table.getHadoopConf(), structType, filter, writeConfig);
+            new HoodieRowParquetWriteSupport(table.getHadoopConf(), structType, bloomFilterOpt, writeConfig);
+
     return new HoodieInternalRowParquetWriter(
-        path, new HoodieRowParquetConfig(
+        path,
+        new HoodieParquetConfig<>(
             writeSupport,
             writeConfig.getParquetCompressionCodec(),
             writeConfig.getParquetBlockSize(),
             writeConfig.getParquetPageSize(),
             writeConfig.getParquetMaxFileSize(),
             writeSupport.getHadoopConf(),
-            writeConfig.getParquetCompressionRatio()));
+            writeConfig.getParquetCompressionRatio(),
+            writeConfig.parquetDictionaryEnabled()
+        ));
   }
 
-  public static HoodieInternalRowFileWriter getInternalRowFileWriterWithoutMetaFields(
-      Path path, HoodieTable hoodieTable, HoodieWriteConfig config, StructType schema)
-      throws IOException {
-    if (PARQUET.getFileExtension().equals(hoodieTable.getBaseFileExtension())) {
-      return newParquetInternalRowFileWriterWithoutMetaFields(path, config, schema, hoodieTable);
+  private static Option<BloomFilter> tryInstantiateBloomFilter(HoodieWriteConfig writeConfig) {
+    // NOTE: Currently Bloom Filter is only going to be populated if meta-fields are populated
+    if (writeConfig.populateMetaFields()) {
+      BloomFilter bloomFilter = BloomFilterFactory.createBloomFilter(
+          writeConfig.getBloomFilterNumEntries(),
+          writeConfig.getBloomFilterFPP(),
+          writeConfig.getDynamicBloomFilterMaxNumEntries(),
+          writeConfig.getBloomFilterType());
+
+      return Option.of(bloomFilter);
     }
-    throw new HoodieIOException(hoodieTable.getBaseFileExtension() + " format not supported yet in row writer path");
-  }
 
-  private static HoodieInternalRowFileWriter newParquetInternalRowFileWriterWithoutMetaFields(
-      Path path, HoodieWriteConfig writeConfig, StructType structType, HoodieTable table)
-      throws IOException {
-    HoodieRowParquetWriteSupport writeSupport =
-        new HoodieRowParquetWriteSupport(table.getHadoopConf(), structType, null, writeConfig);
-    return new HoodieInternalRowParquetWriter(
-        path, new HoodieRowParquetConfig(
-        writeSupport,
-        writeConfig.getParquetCompressionCodec(),
-        writeConfig.getParquetBlockSize(),
-        writeConfig.getParquetPageSize(),
-        writeConfig.getParquetMaxFileSize(),
-        writeSupport.getHadoopConf(),
-        writeConfig.getParquetCompressionRatio()));
+    return Option.empty();
   }
 }
