@@ -38,6 +38,7 @@ import org.apache.hudi.common.testutils.HoodieTestUtils;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.collection.Pair;
 import org.apache.hudi.config.HoodieWriteConfig;
+import org.apache.hudi.exception.HoodieException;
 import org.apache.hudi.keygen.SimpleKeyGenerator;
 import org.apache.hudi.keygen.TimestampBasedKeyGenerator;
 import org.apache.hudi.keygen.constant.KeyGeneratorOptions;
@@ -82,8 +83,10 @@ import static org.apache.hudi.common.table.HoodieTableConfig.TYPE;
 import static org.apache.hudi.common.testutils.HoodieTestDataGenerator.DEFAULT_FIRST_PARTITION_PATH;
 import static org.apache.hudi.common.testutils.HoodieTestDataGenerator.DEFAULT_SECOND_PARTITION_PATH;
 import static org.apache.hudi.common.util.MarkerUtils.MARKERS_FILENAME_PREFIX;
+import static org.apache.hudi.common.util.PartitionPathEncodeUtils.DEPRECATED_DEFAULT_PARTITION_PATH;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 
 /**
@@ -326,10 +329,78 @@ public class TestUpgradeDowngrade extends HoodieClientTestBase {
     assertEquals(checksum, metaClient.getTableConfig().getProps().getString(HoodieTableConfig.TABLE_CHECKSUM.key()));
   }
 
+  @Test
+  public void testUpgradeFourtoFive() throws Exception {
+    testUpgradeFourToFiveInternal(false, false);
+  }
+
+  @Test
+  public void testUpgradeFourtoFiveWithDefaultPartition() throws Exception {
+    testUpgradeFourToFiveInternal(true, false);
+  }
+
+  @Test
+  public void testUpgradeFourtoFiveWithDefaultPartitionWithSkipValidation() throws Exception {
+    testUpgradeFourToFiveInternal(true, true);
+  }
+
+  private void testUpgradeFourToFiveInternal(boolean assertDefaultPartition, boolean skipDefaultPartitionValidation) throws Exception {
+    String tableName = metaClient.getTableConfig().getTableName();
+    // clean up and re instantiate meta client w/ right table props
+    cleanUp();
+    initSparkContexts();
+    initPath();
+    initTestDataGenerator();
+
+    Map<String, String> params = new HashMap<>();
+    addNewTableParamsToProps(params, tableName);
+    Properties properties = new Properties();
+    params.forEach((k,v) -> properties.setProperty(k, v));
+
+    initMetaClient(getTableType(), properties);
+    // init config, table and client.
+    HoodieWriteConfig cfg = getConfigBuilder().withAutoCommit(false).withRollbackUsingMarkers(false)
+        .doSkipDefaultPartitionValidation(skipDefaultPartitionValidation).withProps(params).build();
+    SparkRDDWriteClient client = getHoodieWriteClient(cfg);
+    // Write inserts
+    doInsert(client);
+
+    if (assertDefaultPartition) {
+      doInsertWithDefaultPartition(client);
+    }
+
+    // downgrade table props
+    downgradeTableConfigsFromFiveToFour(cfg);
+
+    // perform upgrade
+    if (assertDefaultPartition && !skipDefaultPartitionValidation) {
+      // if "default" partition is present, upgrade should fail
+      assertThrows(HoodieException.class, () -> new UpgradeDowngrade(metaClient, cfg, context, SparkUpgradeDowngradeHelper.getInstance())
+          .run(HoodieTableVersion.FIVE, null), "Upgrade from 4 to 5 is expected to fail if \"default\" partition is present.");
+    } else {
+      new UpgradeDowngrade(metaClient, cfg, context, SparkUpgradeDowngradeHelper.getInstance())
+          .run(HoodieTableVersion.FIVE, null);
+
+      // verify hoodie.table.version got upgraded
+      metaClient = HoodieTableMetaClient.builder().setConf(context.getHadoopConf().get()).setBasePath(cfg.getBasePath()).build();
+      assertEquals(metaClient.getTableConfig().getTableVersion().versionCode(), HoodieTableVersion.FIVE.versionCode());
+      assertTableVersionFromPropertyFile(HoodieTableVersion.FIVE);
+
+      // verify table props
+      assertTableProps(cfg);
+    }
+  }
+
   private void addNewTableParamsToProps(Map<String, String> params) {
+    addNewTableParamsToProps(params, metaClient.getTableConfig().getTableName());
+  }
+
+  private void addNewTableParamsToProps(Map<String, String> params, String tableName) {
     params.put(KeyGeneratorOptions.RECORDKEY_FIELD_NAME.key(), "uuid");
+    params.put(HoodieTableConfig.RECORDKEY_FIELDS.key(), "uuid");
+    params.put(HoodieTableConfig.PARTITION_FIELDS.key(), "partition_path");
     params.put(KeyGeneratorOptions.PARTITIONPATH_FIELD_NAME.key(), "partition_path");
-    params.put(HoodieTableConfig.NAME.key(), metaClient.getTableConfig().getTableName());
+    params.put(HoodieTableConfig.NAME.key(), tableName);
     params.put(BASE_FILE_FORMAT.key(), BASE_FILE_FORMAT.defaultValue().name());
   }
 
@@ -342,6 +413,16 @@ public class TestUpgradeDowngrade extends HoodieClientTestBase {
     client.insert(writeRecords, commit1).collect();
   }
 
+  private void doInsertWithDefaultPartition(SparkRDDWriteClient client) {
+    // Write 1 (only inserts)
+    dataGen = new HoodieTestDataGenerator(new String[]{DEPRECATED_DEFAULT_PARTITION_PATH});
+    String commit1 = "005";
+    client.startCommitWithTime(commit1);
+    List<HoodieRecord> records = dataGen.generateInserts(commit1, 100);
+    JavaRDD<HoodieRecord> writeRecords = jsc.parallelize(records, 1);
+    client.insert(writeRecords, commit1).collect();
+  }
+
   private void downgradeTableConfigsFromTwoToOne(HoodieWriteConfig cfg) throws IOException {
     Properties properties = new Properties(cfg.getProps());
     properties.remove(HoodieTableConfig.RECORDKEY_FIELDS.key());
@@ -368,6 +449,15 @@ public class TestUpgradeDowngrade extends HoodieClientTestBase {
     metaClient.getTableConfig().setTableVersion(HoodieTableVersion.TWO);
   }
 
+  private void downgradeTableConfigsFromFiveToFour(HoodieWriteConfig cfg) throws IOException {
+    Properties properties = new Properties();
+    cfg.getProps().forEach((k,v) -> properties.setProperty((String) k, (String) v));
+    properties.setProperty(HoodieTableConfig.VERSION.key(), "4");
+    metaClient = HoodieTestUtils.init(hadoopConf, basePath, getTableType(), properties);
+    // set hoodie.table.version to 4 in hoodie.properties file
+    metaClient.getTableConfig().setTableVersion(HoodieTableVersion.FOUR);
+  }
+
   private void assertTableProps(HoodieWriteConfig cfg) {
     HoodieTableConfig tableConfig = metaClient.getTableConfig();
     Properties originalProps = cfg.getProps();
