@@ -18,6 +18,11 @@
 
 package org.apache.hudi.common.model;
 
+import java.util.Arrays;
+import java.util.HashSet;
+import java.util.Set;
+import java.util.stream.Collectors;
+
 /**
  * Hoodie file format.
  */
@@ -27,6 +32,11 @@ public enum HoodieFileFormat {
   HFILE(".hfile"),
   ORC(".orc");
 
+  public static final Set<String> BASE_FILE_EXTENSIONS = Arrays.stream(HoodieFileFormat.values())
+      .map(HoodieFileFormat::getFileExtension)
+      .filter(x -> !x.equals(HoodieFileFormat.HOODIE_LOG.getFileExtension()))
+      .collect(Collectors.toCollection(HashSet::new));
+
   private final String extension;
 
   HoodieFileFormat(String extension) {
