@@ -21,8 +21,10 @@ package org.apache.hudi.table.catalog;
 import org.apache.hudi.common.model.HoodieTableType;
 import org.apache.hudi.configuration.FlinkOptions;
 import org.apache.hudi.exception.HoodieCatalogException;
+import org.apache.hudi.util.StreamerUtil;
 
 import org.apache.flink.table.api.DataTypes;
+import org.apache.flink.table.api.Schema;
 import org.apache.flink.table.api.TableSchema;
 import org.apache.flink.table.catalog.CatalogBaseTable;
 import org.apache.flink.table.catalog.CatalogTable;
@@ -36,7 +38,6 @@ import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hive.metastore.api.Table;
-import org.apache.hudi.util.StreamerUtil;
 import org.junit.jupiter.api.AfterAll;
 import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.BeforeAll;
@@ -50,6 +51,7 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.stream.Collectors;
 
 import static org.apache.flink.table.factories.FactoryUtil.CONNECTOR;
 import static org.junit.jupiter.api.Assertions.assertEquals;
@@ -103,15 +105,42 @@ public class TestHoodieHiveCatalog {
         new CatalogTableImpl(schema, partitions, options, "hudi table");
     hoodieCatalog.createTable(tablePath, table, false);
 
+    // validate hive table
+    Table hiveTable = hoodieCatalog.getHiveTable(tablePath);
+    String fieldSchema = hiveTable.getSd().getCols().stream()
+        .map(f -> f.getName() + ":" + f.getType())
+        .collect(Collectors.joining(","));
+    String expectedFieldSchema = ""
+        + "_hoodie_commit_time:string,"
+        + "_hoodie_commit_seqno:string,"
+        + "_hoodie_record_key:string,"
+        + "_hoodie_partition_path:string,"
+        + "_hoodie_file_name:string,"
+        + "uuid:int,"
+        + "name:string,"
+        + "age:int,"
+        + "ts:bigint";
+    assertEquals(expectedFieldSchema, fieldSchema);
+    String partitionSchema = hiveTable.getPartitionKeys().stream()
+        .map(f -> f.getName() + ":" + f.getType())
+        .collect(Collectors.joining(","));
+    assertEquals("par1:string", partitionSchema);
+
+    // validate catalog table
     CatalogBaseTable table1 = hoodieCatalog.getTable(tablePath);
     assertEquals("hudi", table1.getOptions().get(CONNECTOR.key()));
     assertEquals(tableType.toString(), table1.getOptions().get(FlinkOptions.TABLE_TYPE.key()));
     assertEquals("uuid", table1.getOptions().get(FlinkOptions.RECORD_KEY_FIELD.key()));
     assertNull(table1.getOptions().get(FlinkOptions.PRECOMBINE_FIELD.key()), "preCombine key is not declared");
+    String tableSchema = table1.getUnresolvedSchema().getColumns().stream()
+        .map(Schema.UnresolvedColumn::toString)
+        .collect(Collectors.joining(","));
+    String expectedTableSchema = "`uuid` INT NOT NULL,`name` STRING,`age` INT,`par1` STRING,`ts` BIGINT";
+    assertEquals(expectedTableSchema, tableSchema);
     assertEquals(Collections.singletonList("uuid"), table1.getUnresolvedSchema().getPrimaryKey().get().getColumnNames());
-    assertEquals(Collections.singletonList("par1"), ((CatalogTable)table1).getPartitionKeys());
+    assertEquals(Collections.singletonList("par1"), ((CatalogTable) table1).getPartitionKeys());
 
-    // test explicit primary key
+    // validate explicit primary key
     options.put(FlinkOptions.RECORD_KEY_FIELD.key(), "id");
     table = new CatalogTableImpl(schema, partitions, options, "hudi table");
     hoodieCatalog.alterTable(tablePath, table, true);
