@@ -44,10 +44,10 @@ public class Utils {
         timeCharacteristic,
         processingTimeService,
         new Object(), // no actual locking needed
+        streamTask.getStreamStatusMaintainer(),
         output,
         watermarkInterval,
-        -1,
-        true);
+        -1);
   }
 
   public static FactoryUtil.DefaultDynamicTableContext getTableContext(
