@@ -17,8 +17,8 @@
 
 package org.apache.spark.sql.hudi.procedure
 
-import com.google.common.collect.ImmutableList
 import org.apache.hudi.HoodieSparkUtils
+import org.apache.hudi.common.util.CollectionUtils.createImmutableList
 import org.apache.spark.sql.catalyst.expressions.Literal
 import org.apache.spark.sql.catalyst.plans.logical.{CallCommand, NamedArgument, PositionalArgument}
 import org.apache.spark.sql.hudi.HoodieSparkSqlTestBase
@@ -32,7 +32,7 @@ class TestCallCommandParser extends HoodieSparkSqlTestBase {
 
   test("Test Call Produce with Positional Arguments") {
     val call = parser.parsePlan("CALL c.n.func(1, '2', 3L, true, 1.0D, 9.0e1, 900e-1BD)").asInstanceOf[CallCommand]
-    assertResult(ImmutableList.of("c", "n", "func"))(JavaConverters.seqAsJavaListConverter(call.name).asJava)
+    assertResult(createImmutableList("c", "n", "func"))(JavaConverters.seqAsJavaListConverter(call.name).asJava)
 
     assertResult(7)(call.args.size)
 
@@ -53,7 +53,7 @@ class TestCallCommandParser extends HoodieSparkSqlTestBase {
 
   test("Test Call Produce with Named Arguments") {
     val call = parser.parsePlan("CALL system.func(c1 => 1, c2 => '2', c3 => true)").asInstanceOf[CallCommand]
-    assertResult(ImmutableList.of("system", "func"))(JavaConverters.seqAsJavaListConverter(call.name).asJava)
+    assertResult(createImmutableList("system", "func"))(JavaConverters.seqAsJavaListConverter(call.name).asJava)
 
     assertResult(3)(call.args.size)
 
@@ -64,7 +64,7 @@ class TestCallCommandParser extends HoodieSparkSqlTestBase {
 
   test("Test Call Produce with Var Substitution") {
     val call = parser.parsePlan("CALL system.func('${spark.extra.prop}')").asInstanceOf[CallCommand]
-    assertResult(ImmutableList.of("system", "func"))(JavaConverters.seqAsJavaListConverter(call.name).asJava)
+    assertResult(createImmutableList("system", "func"))(JavaConverters.seqAsJavaListConverter(call.name).asJava)
 
     assertResult(1)(call.args.size)
 
@@ -73,7 +73,7 @@ class TestCallCommandParser extends HoodieSparkSqlTestBase {
 
   test("Test Call Produce with Mixed Arguments") {
     val call = parser.parsePlan("CALL system.func(c1 => 1, '2')").asInstanceOf[CallCommand]
-    assertResult(ImmutableList.of("system", "func"))(JavaConverters.seqAsJavaListConverter(call.name).asJava)
+    assertResult(createImmutableList("system", "func"))(JavaConverters.seqAsJavaListConverter(call.name).asJava)
 
     assertResult(2)(call.args.size)
 
@@ -91,7 +91,7 @@ class TestCallCommandParser extends HoodieSparkSqlTestBase {
 
   test("Test Call Produce with semicolon") {
     val call = parser.parsePlan("CALL system.func(c1 => 1, c2 => '2', c3 => true)").asInstanceOf[CallCommand]
-    assertResult(ImmutableList.of("system", "func"))(JavaConverters.seqAsJavaListConverter(call.name).asJava)
+    assertResult(createImmutableList("system", "func"))(JavaConverters.seqAsJavaListConverter(call.name).asJava)
 
     assertResult(3)(call.args.size)
 
@@ -100,7 +100,7 @@ class TestCallCommandParser extends HoodieSparkSqlTestBase {
     checkArg(call, 2, "c3", true, DataTypes.BooleanType)
 
     val call2 = parser.parsePlan("CALL system.func2(c1 => 1, c2 => '2', c3 => true);").asInstanceOf[CallCommand]
-    assertResult(ImmutableList.of("system", "func2"))(JavaConverters.seqAsJavaListConverter(call2.name).asJava)
+    assertResult(createImmutableList("system", "func2"))(JavaConverters.seqAsJavaListConverter(call2.name).asJava)
 
     assertResult(3)(call2.args.size)
 
