@@ -23,11 +23,10 @@ import org.apache.hadoop.fs.Path
 import org.apache.hudi.common.table.timeline.{HoodieActiveTimeline, HoodieInstant, HoodieTimeline}
 import org.apache.hudi.common.util.{Option => HOption}
 import org.apache.hudi.{HoodieCLIUtils, HoodieDataSourceHelpers}
-import org.apache.spark.sql.hudi.HoodieSparkSqlTestBase
 
 import scala.collection.JavaConverters.asScalaIteratorConverter
 
-class TestClusteringProcedure extends HoodieSparkSqlTestBase {
+class TestClusteringProcedure extends HoodieSparkProcedureTestBase {
 
   test("Test Call run_clustering Procedure By Table") {
     withTempDir { tmp =>
